@@ -398,7 +398,7 @@ static void asm_retf(ASMState *as, IRIns *ir)
   emit_addptr(as, base, -8*delta);
   asm_guard(as, MIPSI_BNE, RID_TMP,
 	    ra_allock(as, i32ptr(pc), rset_exclude(RSET_GPR, base)));
-  emit_tsi(as, MIPSI_LW, RID_TMP, base, -8);
+  emit_tsi(as, MIPSI_LW, RID_TMP, base, LJ_BE ? -8 : -4);
 }
 
 /* -- Type conversions ---------------------------------------------------- */
